#!/home/twinkle/venv/bin/python

import gi

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

from gi.repository import Gtk
from gi.repository import Gdk # GDK_SELECTION_CLIPBOARD

######################################################################
# VARS

MessageType = {
     'GTK_MESSAGE_INFO': Gtk.MessageType.INFO,
  'GTK_MESSAGE_WARNING': Gtk.MessageType.WARNING,
 'GTK_MESSAGE_QUESTION': Gtk.MessageType.QUESTION,
    'GTK_MESSAGE_ERROR': Gtk.MessageType.ERROR,
    'GTK_MESSAGE_OTHER': Gtk.MessageType.OTHER,
}

ButtonType = {
      'GTK_BUTTONS_NONE': Gtk.ButtonsType.NONE,
        'GTK_BUTTONS_OK': Gtk.ButtonsType.OK,
     'GTK_BUTTONS_CLOSE': Gtk.ButtonsType.CLOSE,
    'GTK_BUTTONS_CANCEL': Gtk.ButtonsType.CANCEL,
    'GTK_BUTTONS_YES_NO': Gtk.ButtonsType.YES_NO,
 'GTK_BUTTONS_OK_CANCEL': Gtk.ButtonsType.OK_CANCEL,
}

# Message Type
GTK_MESSAGE_INFO = Gtk.MessageType.INFO
GTK_MESSAGE_WARNING = Gtk.MessageType.WARNING
GTK_MESSAGE_QUESTION = Gtk.MessageType.QUESTION
GTK_MESSAGE_ERROR = Gtk.MessageType.ERROR
GTK_MESSAGE_OTHER = Gtk.MessageType.OTHER

# Button Type
GTK_BUTTONS_NONE = Gtk.ButtonsType.NONE
GTK_BUTTONS_OK = Gtk.ButtonsType.OK
GTK_BUTTONS_CLOSE = Gtk.ButtonsType.CLOSE
GTK_BUTTONS_CANCEL = Gtk.ButtonsType.CANCEL
GTK_BUTTONS_YES_NO = Gtk.ButtonsType.YES_NO
GTK_BUTTONS_OK_CANCEL = Gtk.ButtonsType.OK_CANCEL

######################################################################
# MAIN
if __name__ == "__main__":
    print(f"[{__name__}]")
    print(__doc__)

#=====================================================================
# ALL - Make it directly accessible from the top level of the package
__all__ = ["MessageType", "ButtonType",
    "GTK_MESSAGE_INFO", "GTK_MESSAGE_WARNING", "GTK_MESSAGE_QUESTION", "GTK_MESSAGE_ERROR", "GTK_MESSAGE_OTHER",
    "GTK_BUTTONS_NONE", "GTK_BUTTONS_OK", "GTK_BUTTONS_CLOSE", "GTK_BUTTONS_CANCEL", "GTK_BUTTONS_YES_NO", "GTK_BUTTONS_OK_CANCEL",
]

""" __DATA__

__END__ """



#!/home/twinkle/venv/bin/python

import os
import re
import json
import argparse

import urllib.parse

######################################################################
# LIBS

from sdgp import sdgp
from sdgp.gtk import *
from sdgp.gtk.dialog import dialog
#from sdgpp.gtk.txview import txview

######################################################################
# ArgParse

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', type=str, required=True)
    args = parser.parse_args()

    path = args.i

    path = re.sub('^file://', '', path)
    path = urllib.parse.unquote(path)

    hako = sdgp(path)

    if hako is not None:

        prmp = hako.pop("prompt").replace("<", "&lt;")
        prmp = prmp.replace(">", "&gt;")
        prmp = prmp.replace("\x22", "&quot;")
        prmp = prmp.replace("|", "\\|")
        prmp = re.sub("[\r\n]$", "", prmp)

        ngtv = hako.pop("negativePrompt").replace("<", "&lt;")
        ngtv = ngtv.replace(">", "&gt;")
        ngtv = ngtv.replace("\x22", "&quot;")
        ngtv = ngtv.replace("|", "\\|")
        prmp = re.sub("[\r\n]$", "", prmp)

        hall = f"Prompt: {prmp}\n\nNegative Prompt: {ngtv}\n\n" #json.dumps(hako, indent=4)

        for k in hako.keys():
            hall = hall + f"{k}: {hako[k]}, "

        dialog(hall, 'Stable Diffusion Creation Info', 'sd-get-prompt', GTK_MESSAGE_INFO, GTK_BUTTONS_OK)

######################################################################
# MAIN
if __name__ == "__main__":
    main()

""" __DATA__

__END__ """



#!/home/twinkle/venv/bin/python
# -*- encoding: utf-8 -*-

import gi

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

from gi.repository import Gtk
from gi.repository import Gdk # GDK_SELECTION_CLIPBOARD

######################################################################
# VARS

# Message Type
GTK_MESSAGE_INFO = Gtk.MessageType.INFO
GTK_MESSAGE_WARNING = Gtk.MessageType.WARNING
GTK_MESSAGE_QUESTION = Gtk.MessageType.QUESTION
GTK_MESSAGE_ERROR = Gtk.MessageType.ERROR
GTK_MESSAGE_OTHER = Gtk.MessageType.OTHER

# Button Type
GTK_BUTTONS_NONE = Gtk.ButtonsType.NONE
GTK_BUTTONS_OK = Gtk.ButtonsType.OK
GTK_BUTTONS_CLOSE = Gtk.ButtonsType.CLOSE
GTK_BUTTONS_CANCEL = Gtk.ButtonsType.CANCEL
GTK_BUTTONS_YES_NO = Gtk.ButtonsType.YES_NO
GTK_BUTTONS_OK_CANCEL = Gtk.ButtonsType.OK_CANCEL

######################################################################
# CLASS

def dialog(dtext='Nothing.', label='label', title='dialog', dtype=GTK_MESSAGE_INFO, btype=GTK_BUTTONS_OK):

    if dtype == GTK_MESSAGE_ERROR:
        print(f"{__name__}: {dtext}\n")
    else:
        print(f"{__name__}: {dtext}\n")

    dialog = Gtk.MessageDialog(
                               transient_for=Gtk.Window(),
                               flags=Gtk.DialogFlags.MODAL,
                               message_type=dtype,
                               buttons=btype,
                               text=label
                               )

    dialog.set_default_size(320, 256)
    dialog.set_title("sd-get-prompt")
    dialog.set_skip_taskbar_hint(False);
    dialog.set_position(Gtk.WindowPosition.CENTER);
    dialog.format_secondary_text(dtext);

    # メッセージエリア内のGtkLabelを取得し、選択可能にする
    # GtkMessageDialog.get_message_area() は GtkBox を返す
    # その GtkBox の子ウィジェットが GtkLabel
    message_area = dialog.get_message_area()
    # GtkMessageDialog のメッセージラベルは、通常、message_area の最初の子（主メッセージ）です
    for child in message_area.get_children():
        if isinstance(child, Gtk.Label):
            child.set_selectable(True)
            #break # 最初に見つかったGtkLabelだけを対象とする場合

    retv = dialog.run()

    # クリップボードオブジェクトを取得
    clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
    # クリップボードの内容を永続化（ストア）
    # これにより、アプリケーションが終了してもクリップボードの内容が保持されます
    clipboard.store()

    dialog.destroy()
    # ^^;
    return retv

######################################################################
# MAIN
if __name__ == "__main__":
    print(f"[{__name__}]")
    print(__doc__)

#=====================================================================
# ALL - Make it directly accessible from the top level of the package
__all__ = [
    "dialog", "GTK_MESSAGE_INFO", "GTK_MESSAGE_WARNING", "GTK_MESSAGE_QUESTION", "GTK_MESSAGE_ERROR", "GTK_MESSAGE_OTHER",
    "GTK_BUTTONS_NONE", "GTK_BUTTONS_OK", "GTK_BUTTONS_CLOSE", "GTK_BUTTONS_CANCEL", "GTK_BUTTONS_YES_NO", "GTK_BUTTONS_OK_CANCEL",
]

""" __DATA__

__END__ """



#!/home/twinkle/venv/bin/python
# -*- encoding: utf-8 -*-

import gi

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

from gi.repository import Gtk
from gi.repository import Gdk # GDK_SELECTION_CLIPBOARD
from gi.repository import Pango

######################################################################
# VARS

# Message Type
GTK_MESSAGE_INFO = Gtk.MessageType.INFO
GTK_MESSAGE_WARNING = Gtk.MessageType.WARNING
GTK_MESSAGE_QUESTION = Gtk.MessageType.QUESTION
GTK_MESSAGE_ERROR = Gtk.MessageType.ERROR
GTK_MESSAGE_OTHER = Gtk.MessageType.OTHER

# Button Type
GTK_BUTTONS_NONE = Gtk.ButtonsType.NONE
GTK_BUTTONS_OK = Gtk.ButtonsType.OK
GTK_BUTTONS_CLOSE = Gtk.ButtonsType.CLOSE
GTK_BUTTONS_CANCEL = Gtk.ButtonsType.CANCEL
GTK_BUTTONS_YES_NO = Gtk.ButtonsType.YES_NO
GTK_BUTTONS_OK_CANCEL = Gtk.ButtonsType.OK_CANCEL

######################################################################
# DEFS

def txview(dtext='Nothing.', label='label', title='dialog', dtype=GTK_MESSAGE_INFO, btype=GTK_BUTTONS_OK):

    def on_close(fixed, widget):
        widget.close()

    if dtype == GTK_MESSAGE_ERROR:
        print(f"{__name__}: {dtext}\n")
    else:
        print(f"{__name__}: {dtext}\n")

    gwin = Gtk.Window()

    dialog = Gtk.Dialog(
               title=title,
               transient_for=gwin,
               flags=Gtk.DialogFlags.MODAL,
               )

    dialog.set_default_size(384, 512)
    dialog.set_skip_taskbar_hint(False);
    dialog.set_position(Gtk.WindowPosition.CENTER);

    box = dialog.get_content_area()

    attr = Pango.AttrList()
    attr.insert(Pango.attr_foreground_new(65535, 0, 0))
    attr.insert(Pango.attr_size_new(Pango.SCALE * 48))
    attr.insert(Pango.attr_family_new('Serif'))

    lbl1 = Gtk.Label(label, attr)
    lbl1.set_markup(f"<big><b>{label}</b></big>")
    lbl1.set_margin_top(6)
    lbl1.set_margin_bottom(6)
    box.add(lbl1)

    # GtkTextView を作成し、テキストをセット
    textview = Gtk.TextView()
    textbuffer = textview.get_buffer()
    textbuffer.set_text(dtext)

    # 読み取り専用にする
    textview.set_editable(True)
    textview.set_wrap_mode(True)
    textview.set_cursor_visible(True) # カーソルを非表示に
    textview.set_left_margin(4)
    textview.set_right_margin(4)
    textview.set_top_margin(4)
    textview.set_bottom_margin(4)

    # スクロール可能にするために GtkScrolledWindow に入れる
    scrolled_window = Gtk.ScrolledWindow()
    scrolled_window.set_vexpand(True)
    scrolled_window.set_hexpand(False)
    scrolled_window.add(textview)

    box.pack_start(scrolled_window, True, True, 2)

    btn1 = Gtk.Button("CLOSE")
    btn1.connect("clicked", on_close, dialog)
    box.add(btn1)
    dialog.show_all()

    retv = dialog.run()

    # クリップボードオブジェクトを取得
    clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
    # クリップボードの内容を永続化（ストア）
    # これにより、アプリケーションが終了してもクリップボードの内容が保持されます
    clipboard.store()

    dialog.destroy()
    # ^^;
    return retv

######################################################################
# MAIN
if __name__ == "__main__":
    print(f"[{__name__}]")
    print(__doc__)

#=====================================================================
# ALL - Make it directly accessible from the top level of the package
__all__ = [
    "txview", "GTK_MESSAGE_INFO", "GTK_MESSAGE_WARNING", "GTK_MESSAGE_QUESTION", "GTK_MESSAGE_ERROR", "GTK_MESSAGE_OTHER",
    "GTK_BUTTONS_NONE", "GTK_BUTTONS_OK", "GTK_BUTTONS_CLOSE", "GTK_BUTTONS_CANCEL", "GTK_BUTTONS_YES_NO", "GTK_BUTTONS_OK_CANCEL",
]

""" __DATA__

__END__ """



#!/home/twinkle/venv/bin/python

import json
import re

######################################################################
# VARS

PNGH = b"\x89\x50\x4e\x47\x0d\x0a\x1a\x0a"
IHDR = b"\x00\x00\x00\x0d\x49\x48\x44\x52"

######################################################################
# DEFS

def read_exif(f) -> dict:

    e = {}
    t = ''
    w = 0
    h = 0
    d = 0
    p = ''

    try:
        fh = open(f ,'rb')
    except Exception as e:
        e['_ERROR_'] = e
        return e
    # PNG Header
    t = fh.read(8)
    if(t != PNGH):
        fh.close()
        e['_ERROR_'] = 'Invalid PNG Signature'
        return e
    # IHDR Chunk
    t = fh.read(8)
    if(t != IHDR):
        fh.close()
        e['_ERROR_'] = 'Invalid IHDR'
        return e

    e['PNG'] = 1

    # Width
    t = fh.read(4)
    e['width'] = int.from_bytes(t)
    # Height
    t = fh.read(4)
    e['height'] = int.from_bytes(t)
    # Pixel Depth
    t = fh.read(1)
    e['depth'] = int.from_bytes(t)
    # Color Type
    t = fh.read(1)
    e['color'] = int.from_bytes(t)
    # Compression Method
    t = fh.read(1)
    e['compression'] = int.from_bytes(t)
    # Filter Type
    t = fh.read(1)
    e['filter'] = int.from_bytes(t)
    # Interlace
    t = fh.read(1)
    e['interlace'] = int.from_bytes(t)
    # CRC
    t = fh.read(4)
    e['CRC'] = int.from_bytes(t)

    e['parameters_Pos'] = fh.tell()

    # getChunkSize()
    t = fh.read(4)
    # Size of parameters
    e['parameters_Size'] = int.from_bytes(t);

    # check tEXt(when iTXt on Meitu)
    t = fh.read(4)
    if t != b'tEXt' and t != b'iTXt':
        fh.close()
        e['_ERROR_'] = 'tEXt: Nothing tEXt chunk!('+t.decode('utf-8')+')'
        return e

    # parameters
    p = fh.read(e['parameters_Size'])

    if re.match('parameters', p.decode('utf-8')):
        t = re.sub('^parameters.', '', p.decode('utf-8'))
        s = t.split('\n', 2);
        e['prompt'] = s.pop(0)
        e['negativePrompt'] = s.pop(0)
        r = s[0].split(', ')
        for i in range(len(r)):
            k, v = r[i].split(': ', 1)
            e[k] = v
    elif re.match('generation_data', p.decode('utf-8')):
        t = re.sub('^generation_data.', '', p.decode('utf-8'))
        t = re.sub('[^\w]$', '', t)
        mg = json.loads(t)
        for k in mg.keys():
            e[k] = mg[k]
    else:
        fh.close()
        e['_ERROR_'] = 'tEXt: Not a Stable Diffusion Parameters!'
        return e

    # CRC
    t = fh.read(4)
    e['tEXt_CRC'] = int.from_bytes(t)

    fh.close()

    if len(e) == 0:
        e['_ERROR_'] = 'LEN: Not have a length!'
        return e

    return e

######################################################################
# FRONT

def sdgp(path):

    hako = read_exif(path)

    if '_ERROR_' in hako:
        print(hako['_ERROR_'])
        return None

    hako.pop("PNG")

    return hako

######################################################################
# MAIN
if __name__ == "__main__":
    print(f"[{__name__}]")
    print(__doc__)

#=====================================================================
# ALL - Make it directly accessible from the top level of the package
__all__ = ["sdgp"]

""" __DATA__

__END__ """
